/*******************************

  Palconvert.

  Converts color codes from 
  source file into a pcx that 
  does not use the action colors

  Artistic rights 2004 Korenn

  you can do whatever you want
  with this, just credit me =]

*******************************/
#include <stdio.h>
#include <stdlib.h>

unsigned char pal [256];

int main (int argc, char **argv)
{
	unsigned char * buf;
	int i,filesize,companyswitch, outputname;
	FILE *fp;
	if (argc < 2 || argv[1][1] == '-')
	{
		printf("Converts color codes from source pcx file into a pcx file that does not use the action colors. The file must contain the correct ttd pallette itself.\nIf <target pcx> is omitted, the source file will be overwritten.\n\nUsage:\n\t palconvert <source pcx> [<target pcx>] [options]\n\nOptions:\n\t-C:\tDon't change company colors\n\nExample:\n\tpalconvert somefile.pcx newfile.pcx -C\n\tpalconvert overwritefile.pcx\n");
		return 0;
	}
	// read options

	companyswitch = 0;
	outputname = 1;
	for (i = 2;i < argc; i++)
	{
		if (argv[i][0] == '-')
		{
			switch(argv[i][1])
			{
			case 'C':
				companyswitch = 1;
				break;
			default:
				printf("Invalid switch %s\n", argv[i]);
				return 0;
			}
		}
		else
		{
			if (outputname == 1)
			outputname = i;
			else
			{
				printf("Invalid switch %s\n", argv[i]);
				return 0;
			}
		}
	}
	// read faulty pcx file
	printf("Reading file...\n");
	fp = fopen(argv[1],"rb");
	if (!fp)
	{
		printf("Can't read file %s.\n",argv[1]);
		return 0;
	}
	// get file size
	filesize = 0;
	do
	{
		fgetc(fp);
		if (feof(fp))break;

		filesize++;
	}while (1);
	
	// allocate memory
	buf = (unsigned char *)malloc(filesize);

	if (buf == NULL)
	{
		printf("Not enough memory to process file.\n");
		fclose (fp);
		return 0;
	}

	// read file
	rewind(fp);
	for (i = 0;i < filesize;i++)
	{
		buf[i] = fgetc(fp);
	}

	fclose(fp);

	// not a 256 color image
	if (buf[filesize - 769] != 12)
	{
		printf("File %s is not a 256 color image!\n",argv[1]);
		free(buf);
		return 0;
	}

	// replace action colors in file with correct colors.
	// setup replacement colors
	i = 0;
	for (i = 0; i < 256; i++)
	{
		pal[i] = i;
	}
	// set special colors to correct codes:

		// 198-205 = are all company colours 
		// don't swap with -C
		if (companyswitch == 0)
		{
			pal[198] = 144;
			pal[199] = 145;
			pal[200] = 146;
			pal[201] = 147;
			pal[202] = 148;
			pal[203] = 149;
			pal[204] = 150;
			pal[205] = 151;
		}

		pal[217] = 200; 
		pal[218] = 200; 
		pal[219] = 155; 
		pal[220] = 155; 
		pal[221] = 155; 
		pal[222] = 156; 
		pal[223] = 158; 
		pal[224] = 161; 
		pal[225] = 158; 
		pal[226] = 156; 

		//227-231= brownish fast colour cycle (by TBOT and a litle testing by me :P) 
		pal[227] = 122;
		pal[228] = 73;
		pal[229] = 75;
		pal[230] = 117;
		pal[231] = 79;

		//232-238= red flashing light, are part of the fire on the oil platforms (only in fullscreen, also by TBOT ) 
		pal[232] = 164;
		pal[233] = 185;
		pal[234] = 186;
		pal[235] = 187;
		pal[236] = 188;
		pal[237] = 195;
		pal[238] = 189;

		//239-240= if 239 goes light red 240 goes dark red, if 239 then goes dark red 240 goes light red, (thanks nils) 
		pal[239] = 60;
		pal[240] = 184;

		//244-243-242-241= Yelow flashing in the order they are written down (only fullscreen (thanks marcel))
		pal[241] = 25;
		pal[242] = 26;
		pal[243] = 27;
		pal[244] = 191;

	// parse file
	i = 0;
	i += 128; // skip header
	// pallette at end of file, preceded by '12'
	for (;i < filesize - 769;i ++)
	{
		if ((buf[i]>>6) == 3)	// if this a counter, then take next byte
		{
			i++;
		}
		//swap action colors
		buf[i] = pal[buf[i]];
	}

	// write pcx file
	printf("Writing file...\n");
	fp = fopen(argv[outputname],"wb");
	if (!fp)
	{
		printf("Can't write file %s.\n",argv[outputname]);
		free(buf);
		return 0;
	}
	for (i = 0;i < filesize;i++)
	{
		fputc(buf[i],fp);
	}

	printf("Done.\n");
	fclose(fp);
	free(buf);
	return 0;
}